// *************************************************************************************************
//
// Horde3D
//   Next-Generation Graphics Engine
// --------------------------------------
// Copyright (C) 2006-2009 Nicolas Schulz
//
// This software is distributed under the terms of the Eclipse Public License v1.0.
// A copy of the license may be obtained at: http://www.eclipse.org/legal/epl-v10.html
//
// *************************************************************************************************

#include "utOpenGLES11.h"
#include <cstdlib>
#include <cstring>
#include <string>

using namespace std;


namespace glExt
{
	bool EXT_texture_filter_anisotropic = false;	//iphone
	bool EXT_texture_compression_pvrtc = false;//iphone
	bool EXT_texture_compression_s3tc = false;//pc gfx cards
	bool OES_framebuffer_object = false;	//using EXT if available (only using common subset)

	int	majorVersion = 1, minorVersion = 1;
}


namespace h3dGL
{
#ifdef RENDERER_OPENGL15
// GL 1.3
PFNGLACTIVETEXTUREPROC glActiveTexture = 0x0;
PFNGLCLIENTACTIVETEXTUREPROC glClientActiveTexture = 0x0;
PFNGLSAMPLECOVERAGEPROC glSampleCoverage = 0x0;
PFNGLCOMPRESSEDTEXIMAGE2DPROC glCompressedTexImage2D = 0x0;
PFNGLCOMPRESSEDTEXSUBIMAGE2DPROC glCompressedTexSubImage2D = 0x0;

// GL 1.4
PFNGLPOINTPARAMETERFPROC glPointParameterf = 0x0;
PFNGLPOINTPARAMETERFVPROC glPointParameterfv = 0x0;

// GL 1.5
PFNGLBINDBUFFERPROC glBindBuffer = 0x0;
PFNGLDELETEBUFFERSPROC glDeleteBuffers = 0x0;
PFNGLGENBUFFERSPROC glGenBuffers = 0x0;
PFNGLISBUFFERPROC glIsBuffer = 0x0;
PFNGLBUFFERDATAPROC glBufferData = 0x0;
PFNGLBUFFERSUBDATAPROC glBufferSubData = 0x0;
PFNGLGETBUFFERPARAMETERIVPROC glGetBufferParameteriv = 0x0;


	// GL_OES_framebuffer_object
PFNGLISRENDERBUFFEROESPROC glIsRenderbufferOES = 0x0;
PFNGLBINDRENDERBUFFEROESPROC glBindRenderbufferOES = 0x0;
PFNGLDELETERENDERBUFFERSOESPROC glDeleteRenderbuffersOES = 0x0;
PFNGLGENRENDERBUFFERSOESPROC glGenRenderbuffersOES = 0x0;
PFNGLRENDERBUFFERSTORAGEOESPROC glRenderbufferStorageOES = 0x0;
PFNGLGETRENDERBUFFERPARAMETERIVOESPROC glGetRenderbufferParameterivOES = 0x0;
PFNGLISFRAMEBUFFEROESPROC glIsFramebufferOES = 0x0;
PFNGLBINDFRAMEBUFFEROESPROC glBindFramebufferOES = 0x0;
PFNGLDELETEFRAMEBUFFERSOESPROC glDeleteFramebuffersOES = 0x0;
PFNGLGENFRAMEBUFFERSOESPROC glGenFramebuffersOES = 0x0;
PFNGLCHECKFRAMEBUFFERSTATUSOESPROC glCheckFramebufferStatusOES = 0x0;
PFNGLFRAMEBUFFERRENDERBUFFEROESPROC glFramebufferRenderbufferOES = 0x0;
PFNGLFRAMEBUFFERTEXTURE2DOESPROC glFramebufferTexture2DOES = 0x0;
PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVOESPROC glGetFramebufferAttachmentParameterivOES = 0x0;
PFNGLGENERATEMIPMAPOESPROC glGenerateMipmapOES = 0x0; 
#endif
}  // namespace h3dGL


bool isExtensionSupported( string extString )
{
	size_t pos = 0;
	string extensions = (char *)glGetString( GL_EXTENSIONS );
	
	while( (pos = extensions.find( extString, pos )) != string::npos )
	{
		char c = extensions.c_str()[pos + extString.length()];
		if( c == ' ' || c == '\0' ) return true;
		pos += extString.length();
	}

	return false;
}


void getOpenGLVersion()
{
	string version = (char *)glGetString( GL_VERSION );
	
	size_t pos1 = version.find( "." );
	size_t pos2 = version.find( ".", pos1 + 1 );
	if( pos2 == string::npos ) pos2 = version.find( " ", pos1 + 1 );
	if( pos2 == string::npos ) pos2 = version.length();
	size_t pos0 = version.rfind(" ", pos1);
	if( pos0 == string::npos ) pos0 = 0;
	
	glExt::majorVersion = atoi( version.substr( pos0 + 1, pos1 ).c_str() );
	glExt::minorVersion = atoi( version.substr( pos1 + 1, pos2 ).c_str() );
}


void *platGetProcAddress( const char *funcName )
{
#if defined( PLATFORM_WIN )
	return (void *)wglGetProcAddress( funcName );
#elif defined( PLATFORM_IOS)
	return 0x0;
#elif defined( PLATFORM_MAC )
	CFStringRef functionName = CFStringCreateWithCString( kCFAllocatorDefault, funcName, kCFStringEncodingASCII );
	CFURLRef bundleURL = CFURLCreateWithFileSystemPath(
		kCFAllocatorDefault, CFSTR( "/System/Library/Frameworks/OpenGL.framework" ), kCFURLPOSIXPathStyle, true );
	CFBundleRef bundle = CFBundleCreate( kCFAllocatorDefault, bundleURL );
   
	void *function = CFBundleGetFunctionPointerForName( bundle, functionName );
   
	CFRelease( bundle );
	CFRelease( bundleURL );
	CFRelease( functionName );
   
	return function; 
#else
	return (void *)glXGetProcAddressARB( (const GLubyte *)funcName );
#endif
}


bool initOpenGLExtensions()
{
	bool r = true;
	
	getOpenGLVersion();

#ifdef RENDERER_OPENGL15
	// GL 1.3
	r &= (glActiveTexture = (PFNGLACTIVETEXTUREPROC) platGetProcAddress( "glActiveTexture" )) != 0x0;
	r &= (glClientActiveTexture = (PFNGLCLIENTACTIVETEXTUREPROC) platGetProcAddress( "glClientActiveTexture" )) != 0x0;
	r &= (glSampleCoverage = (PFNGLSAMPLECOVERAGEPROC) platGetProcAddress( "glSampleCoverage" )) != 0x0;
	r &= (glCompressedTexImage2D = (PFNGLCOMPRESSEDTEXIMAGE2DPROC) platGetProcAddress( "glCompressedTexImage2D" )) != 0x0;
	r &= (glCompressedTexSubImage2D = (PFNGLCOMPRESSEDTEXSUBIMAGE2DPROC) platGetProcAddress( "glCompressedTexSubImage2D" )) != 0x0;
	
	// GL 1.4
	r &= (glPointParameterf = (PFNGLPOINTPARAMETERFPROC) platGetProcAddress( "glPointParameterf" )) != 0x0;
	r &= (glPointParameterfv = (PFNGLPOINTPARAMETERFVPROC) platGetProcAddress( "glPointParameterfv" )) != 0x0;

	// GL 1.5
	r &= (glBindBuffer = (PFNGLBINDBUFFERPROC) platGetProcAddress( "glBindBuffer" )) != 0x0;
	r &= (glDeleteBuffers = (PFNGLDELETEBUFFERSPROC) platGetProcAddress( "glDeleteBuffers" )) != 0x0;
	r &= (glGenBuffers = (PFNGLGENBUFFERSPROC) platGetProcAddress( "glGenBuffers" )) != 0x0;
	r &= (glIsBuffer = (PFNGLISBUFFERPROC) platGetProcAddress( "glIsBuffer" )) != 0x0;
	r &= (glBufferData = (PFNGLBUFFERDATAPROC) platGetProcAddress( "glBufferData" )) != 0x0;
	r &= (glBufferSubData = (PFNGLBUFFERSUBDATAPROC) platGetProcAddress( "glBufferSubData" )) != 0x0;
	r &= (glGetBufferParameteriv = (PFNGLGETBUFFERPARAMETERIVPROC) platGetProcAddress( "glGetBufferParameteriv" )) != 0x0;
#endif
	// Extensions
	glExt::OES_framebuffer_object = isExtensionSupported( "GL_OES_framebuffer_object" );
	if( glExt::OES_framebuffer_object )
	{
#ifdef RENDERER_OPENGL15
		r &= (glIsRenderbufferOES = (PFNGLISRENDERBUFFEROESPROC) platGetProcAddress( "glIsRenderbufferOES" )) != 0x0;
		r &= (glBindRenderbufferOES = (PFNGLBINDRENDERBUFFEROESPROC) platGetProcAddress( "glBindRenderbufferOES" )) != 0x0;
		r &= (glDeleteRenderbuffersOES = (PFNGLDELETERENDERBUFFERSOESPROC) platGetProcAddress( "glDeleteRenderbuffersOES" )) != 0x0;
		r &= (glGenRenderbuffersOES = (PFNGLGENRENDERBUFFERSOESPROC) platGetProcAddress( "glGenRenderbuffersOES" )) != 0x0;
		r &= (glRenderbufferStorageOES = (PFNGLRENDERBUFFERSTORAGEOESPROC) platGetProcAddress( "glRenderbufferStorageOES" )) != 0x0;
		r &= (glGetRenderbufferParameterivOES = (PFNGLGETRENDERBUFFERPARAMETERIVOESPROC) platGetProcAddress( "glGetRenderbufferParameterivOES" )) != 0x0;
		r &= (glIsFramebufferOES = (PFNGLISFRAMEBUFFEROESPROC) platGetProcAddress( "glIsFramebufferOES" )) != 0x0;
		r &= (glBindFramebufferOES = (PFNGLBINDFRAMEBUFFEROESPROC) platGetProcAddress( "glBindFramebufferOES" )) != 0x0;
		r &= (glDeleteFramebuffersOES = (PFNGLDELETEFRAMEBUFFERSOESPROC) platGetProcAddress( "glDeleteFramebuffersOES" )) != 0x0;
		r &= (glGenFramebuffersOES = (PFNGLGENFRAMEBUFFERSOESPROC) platGetProcAddress( "glGenFramebuffersOES" )) != 0x0;
		r &= (glCheckFramebufferStatusOES = (PFNGLCHECKFRAMEBUFFERSTATUSOESPROC) platGetProcAddress( "glCheckFramebufferStatusOES" )) != 0x0;
		r &= (glFramebufferTexture2DOES = (PFNGLFRAMEBUFFERTEXTURE2DOESPROC) platGetProcAddress( "glFramebufferTexture2DOES" )) != 0x0;
		r &= (glFramebufferRenderbufferOES = (PFNGLFRAMEBUFFERRENDERBUFFEROESPROC) platGetProcAddress( "glFramebufferRenderbufferOES" )) != 0x0;
		r &= (glGetFramebufferAttachmentParameterivOES = (PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVOESPROC) platGetProcAddress( "glGetFramebufferAttachmentParameterivOES" )) != 0x0;
		r &= (glGenerateMipmapOES = (PFNGLGENERATEMIPMAPOESPROC) platGetProcAddress( "glGenerateMipmapOES" )) != 0x0;
#endif
	}
	else
	{
		glExt::OES_framebuffer_object = isExtensionSupported( "GL_EXT_framebuffer_object" );
		if( glExt::OES_framebuffer_object )
		{
#ifdef RENDERER_OPENGL15
			r &= (glIsRenderbufferOES = (PFNGLISRENDERBUFFEROESPROC) platGetProcAddress( "glIsRenderbufferEXT" )) != 0x0;
			r &= (glBindRenderbufferOES = (PFNGLBINDRENDERBUFFEROESPROC) platGetProcAddress( "glBindRenderbufferEXT" )) != 0x0;
			r &= (glDeleteRenderbuffersOES = (PFNGLDELETERENDERBUFFERSOESPROC) platGetProcAddress( "glDeleteRenderbuffersEXT" )) != 0x0;
			r &= (glGenRenderbuffersOES = (PFNGLGENRENDERBUFFERSOESPROC) platGetProcAddress( "glGenRenderbuffersEXT" )) != 0x0;
			r &= (glRenderbufferStorageOES = (PFNGLRENDERBUFFERSTORAGEOESPROC) platGetProcAddress( "glRenderbufferStorageEXT" )) != 0x0;
			r &= (glGetRenderbufferParameterivOES = (PFNGLGETRENDERBUFFERPARAMETERIVOESPROC) platGetProcAddress( "glGetRenderbufferParameterivEXT" )) != 0x0;
			r &= (glIsFramebufferOES = (PFNGLISFRAMEBUFFEROESPROC) platGetProcAddress( "glIsFramebufferEXT" )) != 0x0;
			r &= (glBindFramebufferOES = (PFNGLBINDFRAMEBUFFEROESPROC) platGetProcAddress( "glBindFramebufferEXT" )) != 0x0;
			r &= (glDeleteFramebuffersOES = (PFNGLDELETEFRAMEBUFFERSOESPROC) platGetProcAddress( "glDeleteFramebuffersEXT" )) != 0x0;
			r &= (glGenFramebuffersOES = (PFNGLGENFRAMEBUFFERSOESPROC) platGetProcAddress( "glGenFramebuffersEXT" )) != 0x0;
			r &= (glCheckFramebufferStatusOES = (PFNGLCHECKFRAMEBUFFERSTATUSOESPROC) platGetProcAddress( "glCheckFramebufferStatusEXT" )) != 0x0;
			r &= (glFramebufferTexture2DOES = (PFNGLFRAMEBUFFERTEXTURE2DOESPROC) platGetProcAddress( "glFramebufferTexture2DEXT" )) != 0x0;
			r &= (glFramebufferRenderbufferOES = (PFNGLFRAMEBUFFERRENDERBUFFEROESPROC) platGetProcAddress( "glFramebufferRenderbufferEXT" )) != 0x0;
			r &= (glGetFramebufferAttachmentParameterivOES = (PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVOESPROC) platGetProcAddress( "glGetFramebufferAttachmentParameterivEXT" )) != 0x0;
			r &= (glGenerateMipmapOES = (PFNGLGENERATEMIPMAPOESPROC) platGetProcAddress( "glGenerateMipmapEXT" )) != 0x0;
#endif
		}
	}

	glExt::EXT_texture_filter_anisotropic = isExtensionSupported( "GL_EXT_texture_filter_anisotropic" );

	glExt::EXT_texture_compression_pvrtc = isExtensionSupported( "GL_IMG_texture_compression_pvrtc" );

	glExt::EXT_texture_compression_s3tc = isExtensionSupported( "GL_EXT_texture_compression_s3tc" );

	return r;
}

namespace h3dGL
{


}